#!/usr/bin/tclsh
###############################################################################
## MySQL/kobazdb converter 2.2.4
## KobazConvert.tcl
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: MySQL AND tcl 8.1
##           my-tcl sql library for tcl
##
##
## PURPOSE: Convert data to and from MySQL tables
##
## COMMANDS:
## All on the same line:
## KobazConvert.tcl < mysql | db > <filename.db> <mysql host> <mysql username> 
##         <mysql password> -table <table> ?options?
## options consist of:
##    -db <name>     -   Database to use   [default KobazScripts]
##    -new           -   Create new table/file
##    -keep          -   Add to table/file without overwriting
##
## Say you have a KobazQuote.db file, and want to put into mysql,
## You type the following all on one line in your eggdrop directory:
## 
## tclsh scripts/KobazConvert.tcl tomysql KobazQuote.db localhost myusername 
##      mypassword -table KobazQuote -new
##
###############################################################################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################

set debug 0

if {[info exists botnick]} {
    putlog "KobazConvert.tcl: This file NOT ment to be run in the bot!"
    return
}

proc putlog {args} {}
proc bind {args} {}
proc timer {args} {}
proc timerexists {args} {}
proc debug {args} {if {$::debug == 1} {puts $args}}

set kobazconfig_botname Convert
source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

kbrequire DatabaseAPI
kbrequire Config
kbrequire SQL-Check

namespace import ::KobazSQL::*
namespace import ::KobazDB::*

if {$argc == "0" || $argc < "7"} {
    if {$argc < "7"} {
        puts "** Not enough args **"
    }

    puts "
KobazScripts Database converter
Usage: convert <type> <file.db> <mysql host> <mysql username> <mysql password> -table <table> ?options?

  types:
   tomysql          Convert from .db to mysql
   todb             Convert from mysql to .db

  options:
   -db    <name>   Database to use.  \[default KobazScripts\]
   -new *default*  Create new db/table and overwrite current one.
   -keep           Add to db/table without overwriting.
"

    return
}

proc connectdb {} {
    global host user pass
    puts "Connecting to $host ..."
    catch {sql_connect $host $user $pass} error
    if {[lsearch $error "sql*"] == "-1"} {
        puts "Connect failed: $error"
        exit
    }
    puts "Connected!"
    return
}

proc selectdb {db} {
    puts "Selecting database: $db"
    catch {sql_selectdb $db} error
    if {$error != "$db"} {
        return -code error "$error"
    }
    return 1
}

proc load_db {file} {
    puts "Loading db..."
    if {[catch {KobazDB::dbRead filename $file} error]} {error $error}
    set numrecs [KobazDB::dbNumRecs filename]
    return $numrecs
}

proc done {{added ""} {total ""}} {
    if {$added != ""} {
        puts "\nAdded $added records. $total total."
    }
    sql_disconnect
    exit
}

proc error {error} {
    puts $error
    sql_disconnect
    exit 1
}

proc charprint {string num} {
    for {set x 0} {$x < $num} {incr x} {
        puts -nonewline $string
    }
    return
}

# Get files and options
set type    [lindex $argv 0]
set file    [lindex $argv 1]
set host    [lindex $argv 2]
set user    [lindex $argv 3]
set pass    [lindex $argv 4]
set options [lrange $argv 5 end]

set table   [lsearch -exact $options "-table"]
if {$table == "-1"} {
    set table Kobaz
} else {
    set table   [lindex $options [expr $table + 1]]
}
set db   [lsearch -exact $options "-db"]
if {$db == "-1"} {
    set db KobazScripts
} else {
    set db   [lindex $options [expr $db + 1]]
}

set new  [lsearch -exact $options "-new"]
set keep [lsearch -exact $options "-keep"]

if {($new != "-1") && ($keep != "-1")} {
    puts "Very silly to use -new and -keep together"
    exit
}

if {$new != "-1"} {set options "-new"}
if {$keep != "-1"} {set options "-keep"}

if {($new == "-1") && ($keep == "-1")} {
    set options "-new"
}

KobazConfig::set_config KobazDatabaseAPI dbtype 1
KobazConfig::set_config KobazDatabaseAPI mysql_database $db

switch -- $options {
    -keep {
        set msg "And add to existing table."
        set msg2 "And add to existing db file."
    }
    -new {
        set msg "And create new db file."
        set msg2 "And create new table."
    }
}
if {![regexp {^tomysql$|^todb$} $type]} {return -code error "KobazConvert.tcl: You must specify only tomysql or todb"}

switch -- $type {
    tomysql {
        puts "Convert from file      --> $file"
        puts "Using mysql db         --> $db"
        puts "Convert to mysql table --> $table"
        puts "$msg"
    }
    todb {
        puts "Convert from mysql table --> $table"
        puts "Using mysql db           --> $db"
        puts "Convert to file          --> $file"
        puts "$msg2"
    }
}

puts -nonewline "Are options correct? (y/n) \[N\]: "
flush stdout
set yesno [gets stdin]
if {![regexp {^y$|^Y$} $yesno]} {return}
puts ""

connectdb

switch -- $type {
    tomysql {
        switch -- $options {
            -new {
		if {[catch {selectdb $db} error]} {
		    puts "Error selecting db: $db: $error"

		    puts "Attempting to create db: $db"
		    catch {sql_exec "CREATE DATABASE $db"} error
		    if {$error != "0"} {
			puts "Error creating database $db: $error"
			done
		    } else {
			puts "Created database: $db"
			selectdb $db
		    }
		}
		
		catch {sql_exec "DROP TABLE $table"} error
		if {$error != "0"} {
		    debug "Error dropping table: $table: $error"
		} else {
		    puts "Dropped table $table"
		}

		set numrecs [load_db $file]
		puts "Loaded! $numrecs total records."
		
                set result [KobazDBAPI::dbCreate $table $table "[dbFields filename]"]
                if {$result == "1"} {
                    puts "Error creating table: $table: $result"
                    done
                } else {
                    puts "Created table: $table"
                }
            }
            -keep {
		if {[catch {selectdb $db} error]} {
		    puts "Error selecting db: $db: $error"
		    done
		}
			
                if {![KobazDBAPI::check_exists $table]} {
                    puts "Error: Table: $table does not exist, use the \"-new\" option"
                    done
		}

		set numrecs [load_db $file]
		puts "Loaded! $numrecs total records."
            }
        }
	
        puts "Processing $numrecs records:\n"
        set count   "0"
        set percent "0"
        set dots    "50"
        set length  [expr $dots + 5]
        set per     [expr double($numrecs) / double($dots)]

        puts -nonewline "\["
        charprint " " $dots
        puts -nonewline "\] 00%"
        flush stdout

        for {set x 0} {($x < $numrecs) && ($percent <= 98)} {incr x; incr count} {
            set add ""
            set result ""
            foreach field [dbFields filename] {
                set result [dbField filename $field $x]
                lappend add $result
            }

            set error [KobazDBAPI::dbAppend $table $add]

            if {$count >= $per} {
                charprint "\b" $length

                incr length -1;
                incr dots -1;

                puts -nonewline "."
                charprint " " $dots

                if {$percent < 9} {
                    puts -nonewline "\] 0${percent}%"
                } else {
                    puts -nonewline "\] ${percent}%"
                }

                flush stdout

                incr percent 2
                set count "0"
            }

            if {$error != "0"} {
                puts "\nError adding entry: $error"
                done
            }
        }
        
        charprint "\b" $length
        charprint "."  [expr $dots - 1]
        puts "\] 100%"

        set recs [KobazDBAPI::dbNumRecs $table]
        done $x $recs
    }

    todb {
	if {[catch {selectdb $db} error]} {
	    puts "Error selecting db: $db: $error"
	    done
	}

        catch {KobazDBAPI::dbFields $table} fields
        if {[regexp {Table.*'.*'.*doesn't.*exist} $fields]} {
            return -code error "$fields";
        }

        switch -- $options {
            -new {if {[catch {dbCreate filename $file $fields} error]} {error $error}}
            -keep {if {[catch {KobazDB::dbRead quotes $file} error]} {error $error}}
        }

        set numrecs [KobazDBAPI::dbNumRecs $table]
        puts "Processing $numrecs records:\n"

        set count   "0"
        set percent "0"
        set dots    "50"
        set length  [expr $dots + 5]
        set per     [expr double($numrecs) / double($dots)]

        puts -nonewline "\["
        charprint " " $dots
        puts -nonewline "\] 00%"
        flush stdout

        for {set x 0} {($x < $numrecs) && ($percent <= 98)} {incr x; incr count} {
            set data [lrange [lindex [sql_getabsoluterec $table $x] 0] 1 end]
            KobazDB::dbAppend filename $data
            
            if {$count >= $per} {
                charprint "\b" $length

                incr length -1;
                incr dots -1;

                puts -nonewline "."
                charprint " " $dots

                if {$percent < 9} {
                    puts -nonewline "\] 0${percent}%"
                } else {
                    puts -nonewline "\] ${percent}%"
                }

                flush stdout

                incr percent 2
                set count "0"
            }
        }

        charprint "\b" $length
        charprint "."  [expr $dots - 1]
        puts "\] 100%"

        KobazDB::dbSort filename [lindex [KobazDB::dbFields filename] 0]

        if {[catch {KobazDB::dbWrite filename} error]} {error $error}
        done $x [KobazDB::dbNumRecs filename]
    }
}

done

